@@ -38,7 +38,8 @@
 #' @author Q. Huynh
 #' @export
 #' @seealso \link{SS2Data} \link{SS2DataMOM}
-SS2MOM <- function(SSdir, nsim = 48, proyears = 50, reps = 1, maxF = 3, seed = 1, interval = 1, pstar = 0.5,
+SS2MOM <- function(SSdir, nsim = 48, proyears = 50, reps = 1, maxF = 3, seed = 1,
+                   interval = 1, pstar = 0.5,
                    Obs = MSEtool::Generic_Obs, Imp = MSEtool::Perfect_Imp, silent = FALSE,
                    Name = "MOM generated by SS2MOM", Source = "No Source provided", ...) {
 
